﻿// Copyright (C) Microsoft Corporation. All Rights Reserved.
// This code released under the terms of the Microsoft Public License
// (Ms-PL, http://opensource.org/licenses/ms-pl.html).

using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;

namespace Delay
{
    /// <summary>
    /// Class that implements a workaround for a bug in the April '10 Silverlight
    /// 4 Toolkit where toggling the IsEnabled state of a MenuItem directly does
    /// not update its visual state correctly.
    /// </summary>
    public class MenuItemIsEnabledWorkaround : ICommand, IDisposable
    {
        /// <summary>
        /// Gets a value indicating whether the workaround is active.
        /// </summary>
        /// <param name="obj">MenuItem instance.</param>
        /// <returns>True if the workaround is active.</returns>
        public static bool GetIsActive(MenuItem obj)
        {
            return (bool)obj.GetValue(IsActiveProperty);
        }

        /// <summary>
        /// Sets a value indicating whether the workaround is active.
        /// </summary>
        /// <param name="obj">MenuItem instance.</param>
        /// <param name="value">True if the workaround should be active</param>
        public static void SetIsActive(MenuItem obj, bool value)
        {
            obj.SetValue(IsActiveProperty, value);
        }

        /// <summary>
        /// Identifies the IsActiveProperty attached DependencyProperty.
        /// </summary>
        public static readonly DependencyProperty IsActiveProperty =
            DependencyProperty.RegisterAttached("IsActive", typeof(bool), typeof(MenuItemIsEnabledWorkaround), new PropertyMetadata(false, OnIsActiveChanged));

        /// <summary>
        /// Handles changes to the IsActiveProperty attached DependencyProperty.
        /// </summary>
        /// <param name="o">Object for which the value changed.</param>
        /// <param name="e">Property change information.</param>
        private static void OnIsActiveChanged(DependencyObject o, DependencyPropertyChangedEventArgs e)
        {
            var menuItem = o as MenuItem;
            if (null == menuItem)
            {
                throw new ArgumentException("The IsActive attached DependencyProperty is only valid for MenuItem instances.");
            }

            if ((bool)e.NewValue)
            {
                // Enable the workaround
                menuItem.Command = new MenuItemIsEnabledWorkaround(menuItem);
            }
            else
            {
                // Disable the workaround (if present)
                var menuItemIsEnabledWorkaround = menuItem.Command as MenuItemIsEnabledWorkaround;
                if (null != menuItemIsEnabledWorkaround)
                {
                    menuItem.Command = null;
                    menuItemIsEnabledWorkaround.Dispose();
                }
            }
        }

        // Stores a reference to the MenuItem instance.
        private readonly MenuItem _menuItem;

        /// <summary>
        /// Initializes a new instance of the MenuItemIsEnabledWorkaround class.
        /// </summary>
        /// <param name="menuItem">MenuItem to apply to.</param>
        public MenuItemIsEnabledWorkaround(MenuItem menuItem)
        {
            _menuItem = menuItem;
            _menuItem.IsEnabledChanged += new DependencyPropertyChangedEventHandler(MenuItem_IsEnabledChanged);
        }

        /// <summary>
        /// Handles changes to the MenuItem's IsEnabledChanged event.
        /// </summary>
        /// <param name="sender">Event source.</param>
        /// <param name="e">Event arguments.</param>
        private void MenuItem_IsEnabledChanged(object sender, DependencyPropertyChangedEventArgs e)
        {
            // Invoke the ICommand CanExecuteChanged event to notify the MenuItem.
            var handler = CanExecuteChanged;
            if (null != handler)
            {
                handler.Invoke(this, EventArgs.Empty);
            }
        }

        /// <summary>
        /// Returns a value indicating whether the MenuItem is enabled.
        /// </summary>
        /// <param name="parameter">Unused.</param>
        /// <returns>True if the MenuItem is enabled.</returns>
        public bool CanExecute(object parameter)
        {
            return _menuItem.IsEnabled;
        }

        /// <summary>
        /// Event that is invoked when the CanExecute value changes.
        /// </summary>
        public event EventHandler CanExecuteChanged;

        /// <summary>
        /// Unused method required by the ICommand interface.
        /// </summary>
        /// <param name="parameter">Unused.</param>
        public void Execute(object parameter)
        {
            // Unused
        }

        /// <summary>
        /// Releases any resources held by the class.
        /// </summary>
        public void Dispose()
        {
            _menuItem.IsEnabledChanged -= new DependencyPropertyChangedEventHandler(MenuItem_IsEnabledChanged);
        }
    }
}
